SystemOrganization addCategory: #FilesystemLink!

!SeasidePlatformSupport class methodsFor: '*FilesystemLink' stamp: 'pmm 7/19/2007 19:28'!
directoriesIn: aPathString
	| directory |
	directory :=FileDirectory default directoryNamed: aPathString.
	^directory directoryNames collect: [ :each |
		directory fullNameFor: each ]! !

!SeasidePlatformSupport class methodsFor: '*FilesystemLink' stamp: 'pmm 7/19/2007 19:16'!
existsFile: aString
	"Squeak filesystem access is gayer than AIDS"
	| directory entry parent |
	directory := FileDirectory on: aString.
	directory exists ifTrue: [ ^true ].
	entry := [ directory directoryEntry ]
		on: Error
		do: [ ^false ].
	entry isDirectory ifTrue: [ ^false ].
	parent := directory containingDirectory.
	^parent fileExists: aString! !

!SeasidePlatformSupport class methodsFor: '*FilesystemLink' stamp: 'pmm 7/19/2007 18:55'!
isDirectory: aString
	^ (FileDirectory on: aString) directoryEntry isDirectory! !

!SeasidePlatformSupport class methodsFor: '*FilesystemLink' stamp: 'pmm 7/19/2007 18:44'!
pathNameDelimiter
	^ FileDirectory pathNameDelimiter! !

WAEntryPoint subclass: #FLFilesystemLink
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FilesystemLink'!

!FLFilesystemLink class methodsFor: 'accessing' stamp: 'pmm 7/19/2007 18:29'!
description
	^ 'Filesystem Link'! !

!FLFilesystemLink methodsFor: 'private' stamp: 'pmm 7/19/2007 20:26'!
asPathSting: aCollectionOfStrings
	^String streamContents: [ :stream |
		stream
			nextPutAll: self basePath.
		self basePath last = $/
			ifFalse: [ stream nextPut: $/ ].
		aCollectionOfStrings do: [ :each |
			stream
				nextPutAll: each;
				nextPut: $/ ] ].! !

!FLFilesystemLink methodsFor: 'accessing' stamp: 'pmm 7/19/2007 18:22'!
configurationComponent
	^FLHandlerEditor new
		link: self;
		yourself! !

!FLFilesystemLink methodsFor: 'private' stamp: 'pmm 7/19/2007 19:19'!
documentAt: pathString filename: filenameString
	^WAResponse
			document: (SeasidePlatformSupport
				contentsOfFile: pathString
				binary: (WAFileLibrary isBinary: filenameString))
			mimeType: (WAFileLibrary mimetypeFor: (filenameString copyAfterLast: $.))! !

!FLFilesystemLink methodsFor: 'private' stamp: 'pmm 7/19/2007 18:54'!
fullPathFor: aCollectionOfStrings
	^String streamContents: [ :stream |
		| delimeter |
		delimeter := SeasidePlatformSupport pathNameDelimiter.
		stream nextPutAll: self root.
		aCollectionOfStrings do: [ :each |
			stream
				nextPut: delimeter;
				nextPutAll: each ] ]
			! !

!FLFilesystemLink methodsFor: 'processing' stamp: 'pmm 7/19/2007 18:35'!
handleRequest: aRequest
	^ self
		responseAt: aRequest url
		ifAbsent: [ self notFound: aRequest url ]! !

!FLFilesystemLink methodsFor: 'testing' stamp: 'pmm 7/19/2007 18:38'!
isInitialized
	^self root isEmptyOrNil not! !

!FLFilesystemLink methodsFor: 'private' stamp: 'pmm 7/19/2007 18:38'!
notFound: aString
	^ WAResponse notFound: aString! !

!FLFilesystemLink methodsFor: 'private' stamp: 'pmm 7/19/2007 19:37'!
render: aOneArgumentBlock
	"convenvience method to render html
	aOneArgumentBlock will be passed an html canvas"
	^String streamContents: [ :stream |
		| newContext docRoot document renderer  |
		document := (WAHtmlStreamDocument new) stream: stream.
		newContext := WARenderingContext new document: document.
		docRoot := WAHtmlRoot context: newContext.
		renderer := WARenderCanvas context: newContext callbacks: Dictionary new.
		docRoot open: document.
		renderer context document stream: stream.
		aOneArgumentBlock value: renderer.
		renderer flush.
		docRoot close: document ]! !

!FLFilesystemLink methodsFor: 'rendering' stamp: 'pmm 7/19/2007 20:30'!
renderDirectory: aString on: html
	html tableRow: [
		html tableData: [
			html anchor
				url: (self webNameOf: aString);
				with: (SeasidePlatformSupport localNameOf: aString), '/' ] ]! !

!FLFilesystemLink methodsFor: 'rendering' stamp: 'pmm 7/19/2007 20:31'!
renderFile: aString on: html
	html tableRow: [
		html tableData: [
			html anchor
				url: (self webNameOf: aString);
				with: (SeasidePlatformSupport localNameOf: aString) ] ]! !

!FLFilesystemLink methodsFor: 'rendering' stamp: 'pmm 7/19/2007 20:27'!
renderHeading: aWebPath on: html
	html heading: (self asPathSting: aWebPath)
	! !

!FLFilesystemLink methodsFor: 'rendering' stamp: 'pmm 7/19/2007 20:34'!
renderListing: aFilesystemPath named: aWebPath
	| html |
	html := self render: [ :canvas |
		self renderHeading: aWebPath on: canvas.
		self renderUp: aWebPath on: canvas.
		canvas table: [
			(SeasidePlatformSupport directoriesIn: aFilesystemPath) do: [ :each |
				self renderDirectory: each on: canvas ].
			(SeasidePlatformSupport filesIn: aFilesystemPath) do: [ :each |
				self renderFile: each on: canvas ] ] ].
	^WAResponse
		document: html
		mimeType: 'text/html'
	! !

!FLFilesystemLink methodsFor: 'rendering' stamp: 'pmm 7/19/2007 20:36'!
renderUp: aWebPath on: html
	aWebPath isEmpty ifTrue: [ ^self ].

	html tableRow: [
		html tableData: [
			html text: '[ '.
			html anchor
				url: (self asPathSting: aWebPath allButLast);
				with: '..'. ].
			html text: ' ]'. ]! !

!FLFilesystemLink methodsFor: 'private' stamp: 'pmm 7/19/2007 19:34'!
responseAt: aPath ifAbsent: aBlock
	| relativePath fullPath |
	self isInitialized ifFalse: [ ^aBlock value ].
	
	relativePath := (aPath allButFirst: self basePath size) findTokens: $/.
	fullPath := (self fullPathFor: relativePath).
	^(SeasidePlatformSupport existsFile: fullPath)
		ifTrue: [
			(SeasidePlatformSupport isDirectory: fullPath)
				ifTrue: [ self renderListing: fullPath named: relativePath ]
				ifFalse: [ self documentAt: fullPath filename: relativePath last ] ]
		ifFalse: [ self notFound: aPath ]! !

!FLFilesystemLink methodsFor: 'accessing' stamp: 'pmm 7/19/2007 18:23'!
root
	^root! !

!FLFilesystemLink methodsFor: 'accessing' stamp: 'pmm 7/19/2007 18:23'!
root: aString 
	root := aString! !

!FLFilesystemLink methodsFor: 'private' stamp: 'pmm 7/19/2007 20:29'!
webNameOf: aDirectoryPath
	^self asPathSting: ((aDirectoryPath allButFirst: self root size) findTokens: SeasidePlatformSupport pathNameDelimiter)
	! !

WAConfigurationEditor subclass: #FLHandlerEditor
	instanceVariableNames: 'link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FilesystemLink'!

!FLHandlerEditor methodsFor: 'accessing' stamp: 'pmm 7/19/2007 18:21'!
link
	^link! !

!FLHandlerEditor methodsFor: 'accessing' stamp: 'pmm 7/19/2007 18:21'!
link: anObject
	link := anObject! !

!FLHandlerEditor methodsFor: 'rendering' stamp: 'pmm 7/19/2007 18:34'!
renderFormOn: html
	html heading level1; with: 'Configuration'.
	html table: [
		html tableRow: [
			html tableHeading: 'Root Directory'.
			html tableData: [
				html textInput
					on: #root of: self link ] ] ]! !
